
#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

ELF_RELOC(R_RISCV_NONE,               0)
ELF_RELOC(R_RISCV_32,                 1)
ELF_RELOC(R_RISCV_64,                 2)
ELF_RELOC(R_RISCV_RELATIVE,           3)
ELF_RELOC(R_RISCV_COPY,               4)
ELF_RELOC(R_RISCV_JUMP_SLOT,          5)
ELF_RELOC(R_RISCV_TLS_DTPMOD32,       6)
ELF_RELOC(R_RISCV_TLS_DTPMOD64,       7)
ELF_RELOC(R_RISCV_TLS_DTPREL32,       8)
ELF_RELOC(R_RISCV_TLS_DTPREL64,       9)
ELF_RELOC(R_RISCV_TLS_TPREL32,       10)
ELF_RELOC(R_RISCV_TLS_TPREL64,       11)
ELF_RELOC(R_RISCV_BRANCH,            16)
ELF_RELOC(R_RISCV_JAL,               17)
ELF_RELOC(R_RISCV_CALL,              18)
ELF_RELOC(R_RISCV_CALL_PLT,          19)
ELF_RELOC(R_RISCV_GOT_HI20,          20)
ELF_RELOC(R_RISCV_TLS_GOT_HI20,      21)
ELF_RELOC(R_RISCV_TLS_GD_HI20,       22)
ELF_RELOC(R_RISCV_PCREL_HI20,        23)
ELF_RELOC(R_RISCV_PCREL_LO12_I,      24)
ELF_RELOC(R_RISCV_PCREL_LO12_S,      25)
ELF_RELOC(R_RISCV_HI20,              26)
ELF_RELOC(R_RISCV_LO12_I,            27)
ELF_RELOC(R_RISCV_LO12_S,            28)
ELF_RELOC(R_RISCV_TPREL_HI20,        29)
ELF_RELOC(R_RISCV_TPREL_LO12_I,      30)
ELF_RELOC(R_RISCV_TPREL_LO12_S,      31)
ELF_RELOC(R_RISCV_TPREL_ADD,         32)
ELF_RELOC(R_RISCV_ADD8,              33)
ELF_RELOC(R_RISCV_ADD16,             34)
ELF_RELOC(R_RISCV_ADD32,             35)
ELF_RELOC(R_RISCV_ADD64,             36)
ELF_RELOC(R_RISCV_SUB8,              37)
ELF_RELOC(R_RISCV_SUB16,             38)
ELF_RELOC(R_RISCV_SUB32,             39)
ELF_RELOC(R_RISCV_SUB64,             40)
ELF_RELOC(R_RISCV_GNU_VTINHERIT,     41)
ELF_RELOC(R_RISCV_GNU_VTENTRY,       42)
ELF_RELOC(R_RISCV_ALIGN,             43)
ELF_RELOC(R_RISCV_RVC_BRANCH,        44)
ELF_RELOC(R_RISCV_RVC_JUMP,          45)
ELF_RELOC(R_RISCV_RVC_LUI,           46)
ELF_RELOC(R_RISCV_RELAX,             51)
ELF_RELOC(R_RISCV_SUB6,              52)
ELF_RELOC(R_RISCV_SET6,              53)
ELF_RELOC(R_RISCV_SET8,              54)
ELF_RELOC(R_RISCV_SET16,             55)
ELF_RELOC(R_RISCV_SET32,             56)
ELF_RELOC(R_RISCV_32_PCREL,          57)
ELF_RELOC(R_RISCV_IRELATIVE,         58)
